TOOL_NAME = "Phase Mixer"

--Renoise Keybinds and menus
-------------------------------
renoise.tool():add_keybinding {
  name = "Global:Tools:"..TOOL_NAME,
  invoke = function()main_toggle()end  
}

renoise.tool():add_menu_entry {
  name = "Main Menu:Tools:Ledger`s Scripts:"..TOOL_NAME,
  invoke = function()main_start()end  
}

--[[
------------------------------------------
--Set up Preferences file
------------------------------------------
--create xml
local options = renoise.Document.create {
  value_a = false,
  value_b = 10,
}
 --assign options-object to .preferences so renoise knows to load and update it with the tool
renoise.tool().preferences = options
------------------------------------------
------------------------------------------
--variable syntax 
--options.value_a.value
--]]

--------------------------------------------------------------------------------
--some basic colors for gui elements
--------------------------------------------------------------------------------
--e.g. For changing vb.views["sample present colour 2"].color when states change
COLOR_GREY = {0x30,0x42,0x42}
COLOR_ORANGE ={0xFF,0x66,0x00}
COLOR_YELLOW = {0xE0,0xE0,0x00}
COLOR_BLUE = {0x50,0x40,0xE0}  
COLOR_RED = {0xEE,0x10,0x10}
COLOR_GREEN = {0x20,0x99,0x20}
COLOR_RED_MILD = {0x90,0x10,0x10}

--Constants holding first index of color-code: Used to identify color in vb.views table i.e.
--if vb.views["button"].color[1] == COLOR_ORANGE_FLAG then --etc
COLOR_GREY_FLAG = COLOR_GREY[1]
COLOR_ORANGE_FLAG = COLOR_ORANGE[1]
COLOR_YELLOW_FLAG = COLOR_YELLOW[1]
COLOR_BLUE_FLAG = COLOR_BLUE[1]  
COLOR_RED_FLAG = COLOR_RED[1]
COLOR_GREEN_FLAG = COLOR_GREEN[1]
COLOR_RED_MILD_FLAG = COLOR_RED_MILD[1]


LUA_COUNTS_FROM_1 = 1

--global variables for gui
local my_dialog = nil 
local vb = nil

local initial_button_count = 0

local timer_running = false


--toggle the tool open and closed (keyboard shortcut start-up)
-------------------------------------------------------------
function main_toggle()
----------------------
 --close dialog if it is open
  if (my_dialog and my_dialog.visible) then 
    my_dialog:close()
    --reset global my_dialog
     my_dialog = nil 
  else --run main
    main()
  end
end

--always open/ restart tool (menu entry start-up)
-------------------------------------------------
function main_start()
---------------------
  if (my_dialog and my_dialog.visible) then 
    my_dialog:close()
    --reset global my_dialog
     my_dialog = nil 
  end
  --run main
  main()
end
----------------------------------------------------------


--------------------------------------------------------------------------------
--helper function : custom status message --prefixes tool name and adds brackets 
--------------------------------------------------------------------------------
local function status(message)
  renoise.app():show_status(TOOL_NAME.." Tool: ("..message..")")
end




------------------------------------
------------------------------------
function main()
------------------------------------
------------------------------------


  --toggle dialog so if its already open then close it.  The timer will catch
  --that it is closed and do the housekeeping like removing notifiers (and timer itself)
  if (my_dialog and my_dialog.visible) then 
    my_dialog:close()
    return
  end
  
  --renoise song object
  local song = renoise.song()

  --set globals
  -------------
  --assign global vb to viewbuilder
  vb = renoise.ViewBuilder()
  
  ----------------------------------------------
  --GUI
  ----------------------------------------------     
  
  local total_trks = #song.tracks
  initial_button_count = total_trks
  --------------------------------
  --column to hold all track buttons
  local button_column = vb:column {margin = 5}
  local my_first_element = nil
  --loop to create "buttons" and add them to my_first_row (buttons are just rows of elements including buttons here)
  for button = 1, total_trks do
    --1 button
    my_first_element = vb:row{
                         margin = 1,
                         --track selection buttons
                         vb:button{
                           width = 25,
                           height = 25,
                           color = COLOR_GREY,
                           id = "trk sel btn"..button,
                           notifier = function()
                            
                             local song = renoise.song()
                             --select track
                             song.selected_track_index = button
                            
                             --select pb device
                             local trk = song:track(button)
                             local num_devs = #trk.devices
                             
                             --select phasebug if found
                             for i = 2,num_devs do
                                if trk:device(i).device_path == "Audio/Effects/VST/PhaseBug" then
                                   song.selected_device_index = i
                                  break
                                end
                             end
                           end,
                          },
                          
                            --spacer
                          vb:text{
                           width = 3,
                          },
                          
                          --invert phase buttons
                         vb:button{
                           width = 25,
                           height = 25,
                           text = "In.",
                           color = COLOR_GREY,
                           id = "invert phase"..button,
                           notifier = function()
                                         local song = renoise.song()
                                         local trk = song:track(button)
                                         local num_devs = #trk.devices
                                         
                                         --check for "Polarity Flipped" and delete if present
                                         local plug_found = false
                                         for i = 2,num_devs do
                                           if (trk:device(i).device_path == "Audio/Effects/Native/Gainer") and
                                             (trk:device(i).display_name == "Polarity Flipped") then
                                             trk:delete_device_at(i)
                                             plug_found = true
                                             break
                                           end
                                         end
                                         
                                         --none found so add gainer instead (acts as toggle)
                                         if plug_found == false then
                                            --add Polarity Gainer
                                            local pol_gain = trk:insert_device_at("Audio/Effects/Native/Gainer", 2)
                                            pol_gain.display_name = "Polarity Flipped"
                                            --hide gain slider in mixer
                                            pol_gain.parameters[1].show_in_mixer = false
                                            --set preset data
                                            pol_gain.active_preset_data = [[<?xml version="1.0" encoding="UTF-8"?>
                                                                <FilterDevicePreset doc_version="13">
                                                                  <DeviceSlot type="GainerDevice">
                                                                    <IsMaximized>false</IsMaximized>
                                                                    <Volume>
                                                                      <Value>1.0</Value>
                                                                    </Volume>
                                                                    <Panning>
                                                                      <Value>0.5</Value>
                                                                    </Panning>
                                                                    <LPhaseInvert>true</LPhaseInvert>
                                                                    <RPhaseInvert>true</RPhaseInvert>
                                                                    <SmoothParameterChanges>true</SmoothParameterChanges>
                                                                  </DeviceSlot>
                                                                </FilterDevicePreset>]]
                                                                
                                           
                                         end
                                     
                                         --run timer
                                         timer()
                                       end,
                          },
                          
                          --spacer
                          vb:text{
                           width = 10,
                          },
                          
                          --enable phasebugs
                          vb:checkbox{
                           width = 25,
                           height = 25,
                            id = "pbug enabled"..button,
                            
                            notifier = function()
                                        
                                       --don't run if timer is calling
                                       if timer_running == true then
                                        return 
                                       end
                          
                                       local song = renoise.song()
                                       local trk = song:track(button)
                                       local num_devs = #trk.devices
                                      
                                       for i = 2,num_devs do
                                         if trk:device(i).device_path == "Audio/Effects/VST/PhaseBug" then
                                            trk:device(i).is_active = not trk:device(i).is_active
                                            return
                                         end
                                       end
                                        
                                       --add phasebug if not found
                                       trk:insert_device_at("Audio/Effects/VST/PhaseBug", 2)
                                     end
                          
                          },
                         
                         --spacer 
                         vb:text{
                           width = 3,
                          },
                          
                         
                          
                         --spacer 
                         vb:text{
                           width = 3,
                          },
                          
                         --track bypass buttons
                         vb:rotary { 
                           height = 25,
                           width = 25,
                           default = 0.58499997854233,
                          -- color = COLOR_GREY,
                          -- text = song:track(button).name,
                           id = "phase rotary"..button,

                           notifier = function(value)
                           
                             if timer_running == true then
                               return
                             end 
                           
                              --song
                              local song = renoise.song()
                              local trk = song:track(button)
                              local num_devs = #trk.devices
                              
                              for i = 2,num_devs do
                                if trk:device(i).device_path == "Audio/Effects/VST/PhaseBug" then
                                  print(trk:device(i):parameter(1).value)
                                  trk:device(i):parameter(1).value = value
                                  trk:device(i):parameter(2).value = value
                                  break
                                end
                                --add phasebug if not found
                                if i == num_devs then
                                  trk:insert_device_at("Audio/Effects/VST/PhaseBug", 2)
                                end
                              end

                            end        
                         },
                         
                         
                         
                          vb:text{
                           
                           text = song:track(button).name,
                           id = "trk name"..button, 
                         
                         },
                        }

    --add each iteration into my_first_row
    button_column:add_child(my_first_element) 
  
  end --end loop
  
  --------------------------------------------------------
  
  
  
  ----main column
  local dialog_content = vb:column{}
  --title text
  local title_text = vb:row{vb:text{ text ="  Trk    Pol.      Phasebug (Allpass)" }}
  
  dialog_content:add_child(title_text)
  --add button column
  dialog_content:add_child(button_column)
  
  --bypass button
  local bypass_row = vb:row{
    margin = 4,
    vb:text{text = "   "},
    vb:button{
       color = COLOR_GREY,
       text = "Bypass",
       notifier = function()
            
            local song = renoise.song()
           
            local active = nil
            
            for tracks = 1,#song.tracks do
              local trk = song:track(tracks)
              local num_devs = #trk.devices
              for i = 2,num_devs do
              if (trk:device(i).device_path == "Audio/Effects/Native/Gainer") and
                 (trk:device(i).display_name == "Polarity Flipped") then
              
              
               -- if trk:device(i).device_path == "Audio/Effects/VST3/ABCDEF019182FAEB72654675466C6970" then
                  --check state of first instance found
                  if active == nil and trk:device(i).is_active == true then
                    active = true
                  elseif active == nil and trk:device(i).is_active == false then
                    active = false
                  end
                  print(active)
                  trk:device(i).is_active = not active
                  
                end
              end
              end
              
              timer()
            end,
    
    },
  }--bypass row
  
  --add bypass row
  dialog_content:add_child(bypass_row)

 
  
  --------------
  --key Handler
  --------------
  local function my_keyhandler_func(dialog,key)
     --toggle lock focus hack, allows pattern ed to get key input
     renoise.app().window.lock_keyboard_focus = not renoise.app().window.lock_keyboard_focus
     renoise.app().window.lock_keyboard_focus = not renoise.app().window.lock_keyboard_focus
    -- rprint(key)
     
     return key
     
  end 
  
 
  ------------------------------------------------------------------------------------------
  ------------------------------------------------------------------------------------------
  --Script dialog
  my_dialog = renoise.app():show_custom_dialog(
      TOOL_NAME, dialog_content,my_keyhandler_func)


  --add timer to fire once every 50ms
  if not renoise.tool():has_timer(timer) then
    renoise.tool():add_timer(timer,80)
  end

  
  --------------------------------------------    
  --close dialog function ON NEW SONG
  --------------------------------------------
  local function closer(d)
    --close dialog if exists and is open     
    if (d ~= nil) and (d.visible == true) then
      d:close()
     -- remove_notifiers()
    end
    --reset global my_dialog
     my_dialog = nil 
  end
  -- notifier to close dialog on load new song
  renoise.tool().app_release_document_observable:add_notifier(closer,my_dialog)
  -------------------------------------------------------------------------------
  
  --first run of timer
   timer()
end


----------------
--timer function
----------------
function timer()
  
  --global flag
  timer_running = true

  local song = renoise.song()
  local selected_track_idx = song.selected_track_index

  --restart tool when user changes track count in renoise (to redraw extra or less buttones)
  -------------------------------------------------------
  if initial_button_count ~= #song.tracks then 
  
    --restart tool function
    -----------------------------
    local function restart_tool() 
    -----------------------------
      --remove_notifiers()
       -- toggle dialog off then on
      if (my_dialog and my_dialog.visible) then 
        my_dialog:close()
        --restart
        main_start()
      end
    end
    --has user changed track count in renoise by adding/ removing tracks?
    --if so then restart/rebuild the tool (easy way to update gui though resets to centre position and causes `flash`)
    restart_tool()
    return
  end
 
  --update track names on text readouts
  -------------------------------
  for track = 1,#song.tracks do
    vb.views["trk name"..track].text = song:track(track).name
  end
  
  local brk_loop = false
  
  --update track selection buttons colors
  ---------------------------------------
  --default all buttons to grey
  for track = 1,#song.tracks do
    vb.views["trk sel btn"..track].color = COLOR_GREY
    --update text [X] to show if current pattern is bypassed
    if song.sequencer:track_sequence_slot_is_muted(track, song.selected_sequence_index) == true then
      vb.views["trk sel btn"..track].text = "X"
    else
      vb.views["trk sel btn"..track].text = ""
    end 
  end
  
  --selected track button to green
  ---------------------------------
  for track = 1,#song.tracks do
    if track == selected_track_idx then
      vb.views["trk sel btn"..track].color = COLOR_GREEN
    end
  end
  
  --Inverted Phase plug
  ---------------------- 
   for track = 1,#song.tracks do
     vb.views["invert phase"..track].color = COLOR_GREY
 
   end
   
   for track = 1,#song.tracks do 
     local num_devs = #song:track(track).devices
     for i = 2,num_devs do
       local dev = song:track(track):device(i)
       --look for polarity gainer
       if (dev.device_path == "Audio/Effects/Native/Gainer") and
          (dev.display_name == "Polarity Flipped") then
       

         if dev.is_active then
           vb.views["invert phase"..track].color = COLOR_ORANGE 
         end
       end
     end
  end
  --phasebug enabled checkboxes
  -----------------------------
  for track = 1,#song.tracks do
    
    local trk = song:track(track)
    local num_devs = #trk.devices
    local pb_found = false
    for i = 2,num_devs do
      if trk:device(i).device_path == "Audio/Effects/VST/PhaseBug" then
       vb.views["pbug enabled"..track].value = trk:device(i).is_active
       pb_found = true
      end
    end
    --no phasebug plug found so disable checkbox
    if pb_found == false then
      vb.views["pbug enabled"..track].value = false
    end
  end

  --update roteries
  -----------------
  for track = 1,#song.tracks do
    --vals
    local trk = song:track(track)
    local num_devs = #trk.devices
    --loop devices
    for i = 2,num_devs do
      if trk:device(i).device_path == "Audio/Effects/VST/PhaseBug" then
       vb.views["phase rotary"..track].value = trk:device(i):parameter(1).value
      end
    end
  end
  
  --update timer flag
  timer_running = false
  
  --remove timer when GUI is closed
  if(my_dialog == nil) or (my_dialog.visible == false) then
    if renoise.tool():has_timer(timer) then
      renoise.tool():remove_timer(timer)
    end
  end
end




